@@ -30,7 +30,7 @@ import logging
 import uuid
 
 from aioconsole import ainput
-from typing import TYPE_CHECKING, Optional, Any
+from typing import TYPE_CHECKING, Optional, Any, Callable, Awaitable, Union
 
 from .errors import AuthException, HTTPException
 
@@ -39,6 +39,7 @@ if TYPE_CHECKING:
 
 log = logging.getLogger(__name__)
 _prompt_lock = asyncio.Lock()
+AnyCallable = Union[Callable, Awaitable]
 
 
 class Auth:
@@ -72,7 +73,7 @@ class Auth:
         try:
             log.info('Running authentication.')
             data = await self.authenticate()
-            self._update(data)
+            self._update_data(data)
         except asyncio.CancelledError:
             pass
 
@@ -91,7 +92,22 @@ class Auth:
                 if e.message_code != 'errors.com.epicgames.bad_request':
                     raise
 
-    def _update(self, data: dict) -> None:
+    def _update_launcher_data(self, data: dict) -> None:
+        self.launcher_access_token = data['access_token']
+        self.launcher_expires_in = data['expires_in']
+        self.launcher_expires_at = self.client.from_iso(data["expires_at"])
+        self.launcher_token_type = data['token_type']
+        self.launcher_refresh_token = data['refresh_token']
+        self.launcher_refresh_expires = data['refresh_expires']
+        self.launcher_refresh_expires_at = data['refresh_expires_at']
+        self.launcher_account_id = data['account_id']
+        self.launcher_client_id = data['client_id']
+        self.launcher_internal_client = data['internal_client']
+        self.launcher_client_service = data['client_service']
+        self.launcher_app = data['app']
+        self.launcher_in_app_id = data['in_app_id']
+
+    def _update_data(self, data: dict) -> None:
         self.access_token = data['access_token']
         self.expires_in = data['expires_in']
         self.expires_at = self.client.from_iso(data["expires_at"])
@@ -106,14 +122,15 @@ class Auth:
         self.app = data['app']
         self.in_app_id = data['in_app_id']
 
-    async def grant_refresh_token(self, refresh_token: str) -> dict:
+    async def grant_refresh_token(self, refresh_token: str,
+                                  auth_token: str) -> dict:
         payload = {
             'grant_type': 'refresh_token',
             'refresh_token': refresh_token
         }
 
         return await self.client.http.account_oauth_grant(
-            auth='basic {0}'.format(self.fortnite_token),
+            auth='basic {0}'.format(auth_token),
             device_id=True,
             data=payload
         )
@@ -159,7 +176,7 @@ class Auth:
         )
 
     async def schedule_token_refresh(self) -> None:
-        subtracted = self.expires_at - datetime.datetime.utcnow()
+        subtracted = self.launcher_expires_at - datetime.datetime.utcnow()
         self.token_timeout = (subtracted).total_seconds() - 300
         await asyncio.sleep(self.token_timeout)
 
@@ -181,12 +198,17 @@ class Auth:
             if self.client.user.party is not None:
                 await self.client.user.party._leave()
 
-            data = await self.grant_refresh_token(self.refresh_token)
-            self.launcher_access_token = data['access_token']
-            self.refresh_token = data['refresh_token']
+            data = await self.grant_refresh_token(
+                self.launcher_refresh_token,
+                self.launcher_token
+            )
+            self._update_launcher_data(data)
 
-            exchange_data = await self.exchange_fortnite_code()
-            self._update(exchange_data)
+            exchange_data = await self.grant_refresh_token(
+                self.refresh_token,
+                self.fortnite_token
+            )
+            self._update_data(exchange_data)
 
             log.debug('Refreshing xmpp session')
             await self.client.xmpp.close()
@@ -309,7 +331,7 @@ class EmailAndPasswordAuth(Auth):
 
         log.info('Exchanging code.')
         data = await self.exchange_launcher_code(data['code'])
-        self.launcher_access_token = data['access_token']
+        self._update_launcher_data(data)
         return data
 
     async def authenticate(self) -> dict:
@@ -334,8 +356,9 @@ class ExchangeCodeAuth(Auth):
 
     Parameters
     ----------
-    exchange_code: :class:`str`
-        The exchange code.
+    exchange_code: Union[:class:`str`, Union[Callable, Awaitable]]
+        The exchange code or a function/coroutine that when called returns
+        the exchange code.
     device_id: Optional[:class:`str`]
         A 32 char hex representing your device.
     launcher_token: Optional[:class:`str`]
@@ -345,18 +368,36 @@ class ExchangeCodeAuth(Auth):
         The fortnite token to use with authentication. You should generally
         not need to set this manually.
     """
-    def __init__(self, exchange_code: str, **kwargs: Any) -> None:
+    def __init__(self, exchange_code: Union[str, AnyCallable],
+                 **kwargs: Any) -> None:
         super().__init__(**kwargs)
         self.exchange_code = exchange_code
+        self.resolved_code = None
+
+    async def resolve(self, code_or_callable: Union[str, AnyCallable]) -> str:
+        if isinstance(code_or_callable, str):
+            return code_or_callable
+
+        elif asyncio.iscoroutinefunction(code_or_callable):
+            res = await code_or_callable()
+        else:
+            res = code_or_callable()
+
+        if not isinstance(res, str):
+            raise TypeError('Return type of callable func/coro must be str')
+
+        return res
 
     @property
     def identifier(self) -> str:
-        return self.exchange_code
+        return self.resolved_code
 
     async def launcher_authenticate(self) -> dict:
         log.info('Exchanging code.')
+        self.resolved_code = await self.resolve(self.exchange_code)
+
         try:
-            data = await self.exchange_launcher_code(self.exchange_code)
+            data = await self.exchange_launcher_code(self.resolved_code)
         except HTTPException as e:
             m = 'errors.com.epicgames.account.oauth.exchange_code_not_found'
             if e.message_code == m:
@@ -366,7 +407,7 @@ class ExchangeCodeAuth(Auth):
 
             raise
 
-        self.launcher_access_token = data['access_token']
+        self._update_launcher_data(data)
         return data
 
     async def authenticate(self) -> dict:
@@ -436,7 +477,7 @@ class DeviceAuth(Auth):
 
             raise
 
-        self.launcher_access_token = data['access_token']
+        self._update_launcher_data(data)
         return data
 
     async def launcher_authenticate(self) -> dict:
@@ -444,7 +485,7 @@ class DeviceAuth(Auth):
         data = await self.exchange_launcher_code(code)
 
         await self.kill_token(self.launcher_access_token)
-        self.launcher_access_token = data['access_token']
+        self._update_launcher_data(data)
         return data
 
     async def authenticate(self) -> dict:
@@ -453,6 +494,36 @@ class DeviceAuth(Auth):
         return await self.exchange_fortnite_code()
 
 
+class RefreshTokenAuth(Auth):
+    """Authenticates by the passed launcher refresh token.
+
+    Parameters
+    ----------
+    refresh_token: :class:`str`
+        A valid launcher refresh token.
+    """
+    def __init__(self, refresh_token: str,
+                 **kwargs: Any) -> None:
+        super().__init__(**kwargs)
+
+        self.refresh_token = refresh_token
+
+    @property
+    def identifier(self) -> str:
+        return self.refresh_token
+
+    async def launcher_authenticate(self) -> dict:
+        data = await self.grant_refresh_token(
+            self.refresh_token,
+            self.launcher_token
+        )
+        self._update_launcher_data(data)
+
+    async def authenticate(self) -> dict:
+        await self.launcher_authenticate()
+        return await self.exchange_fortnite_code()
+
+
 class AdvancedAuth(Auth):
     """Authenticates by the available data in the following order:
 
@@ -479,8 +550,9 @@ class AdvancedAuth(Auth):
         The two factor code to use for the login if needed. If this is
         not passed but later needed, you will be prompted to enter it
         in the console.
-    exchange_code: Optional[:class:`str`]
-        The exchange code to use to login.
+    exchange_code: Optional[Union[:class:`str`, Union[Callable, Awaitable]]]
+        The exchange code or a function/coroutine that when called returns
+        the exchange code.
     device_id: Optional[:class:`str`]
         The device id to use for the login.
     account_id: Optional[:class:`str`]
@@ -511,7 +583,7 @@ class AdvancedAuth(Auth):
     def __init__(self, email: Optional[str] = None,
                  password: Optional[str] = None,
                  two_factor_code: Optional[int] = None,
-                 exchange_code: Optional[str] = None,
+                 exchange_code: Optional[Union[str, AnyCallable]] = None,
                  device_id: Optional[str] = None,
                  account_id: Optional[str] = None,
                  secret: Optional[str] = None,
@@ -582,10 +654,10 @@ class AdvancedAuth(Auth):
         auth.initialize(self.client)
 
         data = await auth.device_auth_authenticate()
-        self.launcher_access_token = data['access_token']
+        self._update_launcher_data(data)
 
         data = await auth.launcher_authenticate()
-        self.launcher_access_token = data['access_token']
+        self._update_launcher_data(data)
 
         return await auth.exchange_fortnite_code()
 
@@ -638,10 +710,10 @@ class AdvancedAuth(Auth):
                         loop=self.client.loop
                     )
 
-                data = await self.run_exchange_code_authenticate()
+            data = await self.run_exchange_code_authenticate()
 
         client_id = data['account_id']
-        self.launcher_access_token = data['access_token']
+        self._update_launcher_data(data)
 
         if self.delete_existing_device_auths:
             tasks = []
@@ -671,5 +743,5 @@ class AdvancedAuth(Auth):
         data = await self.exchange_launcher_code(code)
 
         await self.kill_token(self.launcher_access_token)
-        self.launcher_access_token = data['access_token']
+        self._update_launcher_data(data)
         return await self.exchange_fortnite_code()
